.barcode-button
{
  background-color: rgba(255, 255, 255, 1.0);
  padding: 0;
  box-shadow: 0 0 0 1px rgba(0,0,0,0.03), 0 1px 3px 1px rgba(0,0,0,0.07), 0 2px 6px 2px rgba(0,0,0,0.03);
}

.barcode-button:hover
{
  background-color: rgba(245, 245, 245, 0.9);
}

.barcode-button:active
{
  background-color: rgba(215, 215, 215, 0.9);
}

/* Use light background color in barcode dialogue. */
.barcode-dialog
{
  color: rgba(0, 0, 0, 0.8); /* @window_fg_color of Adwaita's light variant */
  background-color: white;
}

/* Do not highlight the selected menu row when the leaflet is folded. */
navigation-view row:selected:not(:hover):not(:active)
{
  background-color: transparent;
}

/* This is a workaround for the following issue: "Placeholder causes additional
   divider on boxed list" (https://gitlab.gnome.org/GNOME/gtk/-/issues/5309). */
.list-box-with-placeholder > row:nth-last-child(2)
{
  border-bottom: 0;
}

.rounded
{
  border-radius: 6px;
}
